//************************************************************
//  Html/app.js
//  Peppermint
//
//  Coded by Dr.Kameleon
//
//  Copyright (c) 2011-2014 InSili.co.
//  All rights reserved.
//************************************************************

/*******************************************************
 *
 * Peppermint Core
 *
 *******************************************************/

var Peppermint = function()
{
	//var editor;
	var self = this;

	this.documentStates = {};

	/*********************************
	 Initialisation
	 *********************************/

	// Main setup
	
	this.editor = ace.edit("editor");
	this.editor.setTheme("ace/theme/peppermint");
	this.editor.getSession().setMode("ace/mode/php");
	this.editor.setOptions({
		enableBasicAutocompletion: true,
		enableSnippets: true,
		enableLiveAutocompletion: true
	});

	// Attach event observers

	this.editor.selection.on("changeCursor", function(e) { 
		var cursor = self.editor.selection.getCursor();

		if (typeof core === 'undefined') console.log(cursor);
		else core.currentLocationChanged_("Line " + (cursor.row+1) + ", Column " + (cursor.column+1));
	});

	this.editor.on("guttermousedown", function(e){ 
		var target = e.domEvent.target; 
		if (target.className.indexOf("ace_gutter-cell") == -1) return; 
		if (!self.editor.isFocused()) return; 
		if (e.clientX > 25 + target.getBoundingClientRect().left) return; 

		var row = e.getDocumentPosition().row;

		if (self.editor.session.$breakpoints[row]=="ace_breakpoint")
			self.bookmarkUnmarkLine(row);
		else
			self.bookmarkMarkLine(row);

		e.stop();
	}); 

	/*********************************
	 Interface
	 *********************************/

	// Commands

	this.exec = function(cmd,args) {
		if (typeof args === 'undefined')
		{
			var argsArr = cmd.split(" ");

			if (argsArr.length==1)
			{
				core.execute_args_(cmd, JSON.stringify([]));
			}
			else
			{
				var cmd = argsArr[0];
				argsArr.shift();
				core.execute_args_(cmd,JSON.stringify(argsArr));
			}
		}
		else
		{
			core.execute_args_(cmd, JSON.stringify(args));
		}
	}

	this.echo = function(message) {
		var msg_lines = message.split("\n");

		for (var m=0; m<msg_lines.length; m++)
		{
			if (msg_lines[m].trim() != "")
				terminal.echo(" % " + msg_lines[m]);
			else
				terminal.echo("");
		}
	}

	this.getFullPath = function()
	{
		return core.getFullPath();
	}

	this.setContent = function(content) {
		self.editor.getSession().setValue(content);
	}

	this.setUndoableContent = function(content) {
		self.editor.setValue(content);
	}

	this.getContent = function() {
		return self.editor.getSession().getValue();
	}

	this.setMode = function(mode) {
		self.editor.getSession().setMode("ace/mode/"+mode);
	}

	this.getMode = function() {
		return self.editor.getSession().getMode().$id.replace("ace/mode/","");
	}

	this.setCursor = function(column,row) {
		self.editor.moveCursorToPosition({row:row, column:column});
	}

	this.getCursor = function() {
		return self.editor.selection.getCursor();
	}

	this.togglePanel = function (onoff) {
		if (onoff) mainLayout.show("south");
		else mainLayout.hide("south");
	}

	this.toggleGutter = function (onoff) {
		self.editor.renderer.setShowGutter(onoff);
	}

	this.togglePrintMargin = function (onoff) {
		self.editor.setShowPrintMargin(onoff);
	}

	this.toggleIndentationGuides = function (onoff) {
		self.editor.renderer.setDisplayIndentGuides(onoff);
	}

	this.toggleInvisibles = function (onoff) {
		self.editor.setShowInvisibles(onoff);
	}

	this.toggleSoftTabs = function (onoff) {
		self.editor.setUseSoftTabs(onoff);
	}

	this.toggleWordWrap = function (onoff) {
		self.editor.getSession().setUseWrapMode(onoff);
	}

	this.setTabSize = function (n) {
		self.editor.getSession().setTabSize(n);
	}

	this.setWrapLimit = function (n) {
		self.editor.getSession().setWrapLimitRange(n,n);
	}

	this.setFontSize = function (n) {
		$("#editor").css("font-size",n+"px");
	}

	this.setTheme = function (t) {
		self.editor.setTheme("ace/theme/"+t);
	}

	this.find = function () {
		var initialPos = pp.getCursor();

		var total = self.editor.findAll( 
			$("#search-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);

		pp.setCursor(initialPos.column, initialPos.row);

		self.editor.find( 
			$("#search-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);

		if (total>0)
			$("#matches-found").html(total + " matches found<!--&nbsp;<a href='#'><i class='glyphicon glyphicon-chevron-up'></i></a>-->");
		else
			$("#matches-found").html("Unable to find any match");

		return total;
	}

	this.findNext = function () {
		self.editor.findNext();
	}

	this.findPrevious = function () {
		self.editor.findPrevious();
	}

	this.useSelectionForFind = function() {
		$("#search-input").val(self.getSelection());
	}

	this.replace = function () {
		self.editor.replace(
			$("#replace-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);
	}

	this.replaceAll = function () {
		self.editor.replaceAll(
			$("#replace-input").val(),
			{
				backwards: false,
				wrap: true,
				regExp: $("#search-opt-regex").hasClass("active"),
				caseSensitive: $("#search-opt-casesensitive").hasClass("active"),
				wholeWord: $("#search-opt-wholeword").hasClass("active")
			}
		);
	}

	this.scrollPageUp = function() {
		self.editor.scrollPageUp();
	}

	this.scrollPageDown = function() {
		self.editor.scrollPageDown();
	}

	this.scrollToSelection = function() {
		self.editor.renderer.scrollSelectionIntoView();
	}

	this.bookmarkMarkLine = function(line) {
		self.editor.session.setBreakpoint(line,'ace_breakpoint');
	}

	this.bookmarkMarkCurrent = function() {
		var curs = self.editor.selection.getCursor();
		self.bookmarkMarkLine(curs.row);
	}

	this.bookmarkUnmarkLine = function(line) {
		self.editor.session.clearBreakpoint(line);
	}

	this.bookmarkUnmarkCurrent = function() {
		var curs = self.editor.selection.getCursor();
		self.bookmarkUnmarkLine(curs.row);
	}

	this.bookmarkRemoveAll = function() {
		self.editor.session.clearBreakpoints();
	}

	this.foldAll = function() {
		self.editor.getSession().foldAll();
	}

	this.unfoldAll = function() {
		self.editor.getSession().expandFolds(self.editor.getSession().getAllFolds());
	}

	this.foldCurrent = function() {
		self.editor.getSession().toggleFold(false);
	}

	this.unfoldCurrent = function() {
		self.editor.getSession().toggleFold(true);
	}

	this.toggleComments = function() {
		self.editor.toggleCommentLines();
	}

	this.goToLine = function(line) {
		self.editor.gotoLine(line);
	}

	this.selectWord = function() {
		self.editor.selection.selectWord();
	}

	this.selectLine = function() {
		self.editor.selection.selectLine();
	}

	this.selectLineEnd = function() {
		self.editor.selection.selectLineEnd();
	}

	this.selectLineStart = function() {
		self.editor.selection.selectLineStart();
	}

	this.selectFileStart = function() {
		self.editor.selection.selectFileStart();
	}

	this.selectFileEnd = function() {
		self.editor.selection.selectFileEnd();
	}

	this.selectAll = function() {
		self.editor.selectAll();
	}

	this.deselectAll = function() {
		self.editor.clearSelection();
	}

	this.indent = function() {
		self.editor.blockIndent();
	}

	this.outdent = function() {
		self.editor.blockOutdent();
	}

	this.moveUp = function() {
		self.editor.moveLinesUp();
	}

	this.moveDown = function() {
		self.editor.moveLinesDown();
	}

	this.noOfUndoItems = function() {
		self.editor.getSession().getUndoManager().$undoStack.length;
	}

	this.noOfRedoItems = function() {
		self.editor.getSession().getUndoManager().$redoStack.length;
	}

	this.resizeEditor = function() {
		self.editor.resize();
	}

	this.undo = function() {
		self.editor.undo();
	}

	this.redo = function() {
		self.editor.redo();
	}

	this.getSelection = function() {
		return self.editor.session.getTextRange(self.editor.getSelectionRange());
	}

	this.setSelection = function(replacement) {
		self.editor.getSession().replace(self.editor.getSelectionRange(),replacement);
	}

	this.getNoOfLines = function() {
		return self.editor.session.getLength();
	}

	this.getWordAtLocation = function() {
		return self.editor.session.getTextRange(self.editor.session.getWordRange(self.editor.selection.getCursor().row,self.editor.selection.getCursor().column));
	}

	this.getTempFileWithExtension = function(ext) {
		return core.getTempFile_withExtension_(JSON.stringify([self.getContent()]), ext);
	}

	this.doRun = function() {
		// call Obj-C to invoke "Run"
	}

	// State handling

	function changer() {
		core.currentDocumentChanged();
	}

	this.getFoldsState = function() {
		return self.editor.getSession().getAllFolds();
	}

	this.setFoldsState = function(foldsState) {
		self.editor.getSession().addFolds(foldsState);
	}

	this.getSelectionState = function() {
		return self.editor.getSession().selection.getRange();
	}

	this.setSelectionState = function(selectionState) {
		self.editor.getSession().selection.setRange(selectionState);
	}

	this.getBreakpointsState = function() {
		return self.editor.getSession().getBreakpoints();
	}

	this.setBreakpointsState = function(breakpointsState) {
		var indices = [];
		for (var i=0; i<breakpointsState.length; i++)
		{
			if (breakpointsState[i] === "ace_breakpoint")
				indices.push(i);
		}

		self.editor.getSession().setBreakpoints(indices);
	}

	this.getAnnotationsState = function() {
		return self.editor.getSession().getAnnotations();
	}

	this.setAnnotationsState = function(annotationsState) {
		self.editor.getSession().setAnnotations(annotationsState);
	}

	this.getUndoState = function() {
		return self.editor.getSession().getUndoManager().$undoStack;
	}

	this.setUndoState = function(undoState) {
		self.editor.getSession().getUndoManager().$undoStack = undoState;
	}

	this.getRedoState = function() {
		return self.editor.getSession().getUndoManager().$redoStack;
	}

	this.setRedoState = function(redoState) {
		self.editor.getSession().getUndoManager().$redoStack = redoState;
	}

	this.saveLocalState = function(docpointer) {
		self.documentStates[docpointer] = {};

		self.documentStates[docpointer].folds = self.getFoldsState();
		self.documentStates[docpointer].selection = self.getSelectionState();
		self.documentStates[docpointer].breakpoints = self.getBreakpointsState();
		self.documentStates[docpointer].annotations = self.getAnnotationsState();
		self.documentStates[docpointer].undo = self.getUndoState();
		self.documentStates[docpointer].redo = self.getRedoState();
	}

	this.loadLocalState = function(docpointer) {
		if (typeof this.documentStates[docpointer] !== 'undefined')
		{
			var st = this.documentStates[docpointer];

			self.setFoldsState(st.folds);
			self.setSelectionState(st.selection);
			self.setBreakpointsState(st.breakpoints);
			self.setAnnotationsState(st.annotations);
			self.setUndoState(st.undo);
			self.setRedoState(st.redo);
		}
	}

	this.setState = function(state) {
		this.setContent(state.content);
		this.setMode(state.mode);

		this.editor.session.on("change", changer);

		this.editor.focus();
	}

	this.getState = function() {
		var result = {
			content : this.getContent(),
			mode 	: this.getMode()
		};

		this.editor.getSession().removeListener('change', changer);

		//this.editor.getSession().removeAllListeners("change");

		return JSON.stringify(result);
	}

	/*********************************
	 Bindings
	 *********************************/

	$("#search-input").on("input",self.find);
	$("#search-left").click(self.findPrevious);
	$("#search-right").click(self.findNext);

	$("#replace-input").bind('keypress', function(e) 
	{
		var code = e.keyCode || e.which;
		if (code==13)
		{
			self.replace();
		}
	});

	$("#replace-one").click(self.replace);
	$("#replace-all").click(self.replaceAll);

	$("#search-opt-regex").click(function(){
		core.searchOptChanged_withValue_("PP_Search_Regex",!$("#search-opt-regex").hasClass("active"));
	});
	$("#search-opt-casesensitive").click(function(){
		core.searchOptChanged_withValue_("PP_Search_Case",!$("#search-opt-casesensitive").hasClass("active"));
	});
	$("#search-opt-wholeword").click(function(){
		core.searchOptChanged_withValue_("PP_Search_Whole_Word",!$("#search-opt-wholeword").hasClass("active"));
	});

	this.searchOptRegexToggle = function(onoff)
	{
		if (onoff) $("#search-opt-regex").addClass('active');
		else $("#search-opt-regex").removeClass('active');
	}

	this.searchOptCaseToggle = function(onoff)
	{
		if (onoff) $("#search-opt-casesensitive").addClass('active');
		else $("#search-opt-casesensitive").removeClass('active');
	}

	this.searchOptWholeWordToggle = function(onoff)
	{
		if (onoff) $("#search-opt-wholeword").addClass('active');
		else $("#search-opt-wholeword").removeClass('active');
	}

	this.showConsoleTab = function()
	{
		$(".panel-tabs li:first a").tab("show");
	}

	this.showSearchTab = function()
	{
		$(".panel-tabs li:last a").tab("show");
	}
}

/*******************************************************
 *
 * Initial Setup
 *
 *******************************************************/

$(function(){

	/*********************************
	 Setup Layout
	 *********************************/

	resizeEditorOnLayoutChange = function()
	{
		pp.resizeEditor();
	}

	window.mainLayout = $("body").layout({
		fxName : "none",
		fxSpeed : "fast",
		slidable: false,
		togglerLength_closed: 0,
		spacing_open: 3,
		spacing_closed: 3,

		onresize_start: resizeEditorOnLayoutChange,
		onresize_end: resizeEditorOnLayoutChange,

		default: {
			onresize_start: resizeEditorOnLayoutChange,
			onresize_end: resizeEditorOnLayoutChange
		},

		panes: {
			liveContentResizing:   true,
			livePaneResizing: 		true
		},

		south : {
			initClosed: true,
			initHidden: true,
			onclose_end: function(){
				$(".panel-tabs li").each(function(){
					$(this).removeClass("active");
				});
				resizeEditorOnLayoutChange();
				mainLayout.hide("south");

				core.panelToggled_(false); // Notify the core, so the it updates the Options + Menus
			},
			closable: true,
			resizable: true,
			onresize_start: resizeEditorOnLayoutChange,
			onresize_end: function(e){
				resizeEditorOnLayoutChange();
				 $("#console-output").css("width",$("#console-input-group").css("width"));
			},
			onopen: function(){
				
				if ($("#console-tab").hasClass("active")) { $("#console-trigger").removeClass("active").addClass("active"); terminal.focus(); }
				if ($("#search-tab").hasClass("active")) { $("#search-trigger").removeClass("active").addClass("active"); }
				core.panelToggled_(true);
			},
			onhide_end: resizeEditorOnLayoutChange,
			size: 110,
			minSize: 110,
			maxSize: $(window).height() - 150
		}
	});

	$(window).resize(function(){
		mainLayout.south.options.maxSize = $(window).height() - 150;
		mainLayout.resizeAll();
	});

	/*********************************
	 Terminal
	 *********************************/

	window.terminalMode = "js";
	window.terminalPrompt = "$> ";
	window.terminal = $('#termem').terminal(
		function(cmd,term)
		{
			switch (cmd)
			{
				case 'version': term.echo("Version: 01/462.1\n"); break;
				case 'help': term.echo("23 available commands\n"); break;
				case 'quit': if (terminalMode=="shell") { terminalMode = "js"; term.set_prompt("$> "); } break;
				case 'shell': term.set_prompt("shell> "); terminalMode = "shell"; break;
				default: 
				{
					if (terminalMode=="js")
					{
						try {
							var rez=eval(cmd);
							term.echo(" = " + rez + "\n");
						}
						catch (e)
						{
							term.echo("[[;red;] ! ]" + e + "\n");
						}
					}
					else if (terminalMode=="shell")
					{
						pp.exec(cmd);
					}
				}
			}
		},
		/*
		{
			version: function() { this.echo("Version: 01/462.1\n"); },
			help: function() { this.echo("23 available commands\n"); },
		    shell : function(){
		    	this.push(function(cmd,term)
				{
					pp.exec(cmd);
				});
		    }
		},*/
		{ 
			prompt: '[[b;;]$>] ', 
			greetings: "[[;#A5E12C;]Peppermint][[b;#65A10C;]/01]\n"
		}
	);

	/*********************************
	 Panel
	 *********************************/

	$("#console-trigger").click(function(){ $(".panel-tabs li:first a").tab("show"); });
	$("#search-trigger").click(function(){ $(".panel-tabs li:last a").tab("show"); });

	$('.panel-tabs a[data-toggle="tab"]').on('show.bs.tab', function (e) {
		if (mainLayout.state.south.isClosed) mainLayout.open("south");
		console.log(e.target.hash);

		if (e.target.hash=="#console-tab") terminal.focus();
	});

	//$("#run-trigger").click(function(){
	//	pp.doRun();
	//});

	/*********************************
	 Initialise Peppermint
	 *********************************/

	 window.pp = new Peppermint();
});
